// Copyright 2021 PingCAP, Inc. Licensed under Apache-2.0.

package build

import (
	"bytes"
	"fmt"
	"runtime"

	"github.com/pingcap/log"
	"go.uber.org/zap"
)

// Version information, generated by build flags in Makefile.
var (
	ReleaseVersion = ""
	BuildTS        = ""
	GitHash        = ""
	GitBranch      = ""
	goVersion      = runtime.Version()
)

// AppName is a name of a built binary.
type AppName string

var (
	// BR is the name of BR binary.
	BR AppName = "Backup & Restore (BR)"
)

// LogInfo logs version information.
func LogInfo(name AppName) {
	oldLevel := log.GetLevel()
	log.SetLevel(zap.InfoLevel)
	defer log.SetLevel(oldLevel)

	log.Info(fmt.Sprintf("Welcome to %s", name),
		zap.String("release-version", ReleaseVersion),
		zap.String("git-hash", GitHash),
		zap.String("git-branch", GitBranch),
		zap.String("go-version", goVersion),
		zap.String("utc-build-time", BuildTS),
		zap.Bool("race-enabled", false))
}

// Info returns version information.
func Info() string {
	buf := bytes.Buffer{}
	fmt.Fprintf(&buf, "Release Version: %s\n", ReleaseVersion)
	fmt.Fprintf(&buf, "Git Commit Hash: %s\n", GitHash)
	fmt.Fprintf(&buf, "Git Branch: %s\n", GitBranch)
	fmt.Fprintf(&buf, "Go Version: %s\n", goVersion)
	fmt.Fprintf(&buf, "UTC Build Time: %s\n", BuildTS)
	fmt.Fprintf(&buf, "Race Enabled: %t", false)
	return buf.String()
}
